# !/usr/bin/python
# -*- coding: utf-8 -*-

# Glitter Toolkit

__authors__ = ["Jan Jokela <janjokela@gmail.com>"]
__licenses__ = ["LICENSE.LGPL"]
__description__ = "Button widget" 

import gobject
import clutter

from container import Container
from image import Image
from label import Label

class Button(Container):
    
    """ A Button widget """ 
    
    def __init__(self, text="Button", animate=True, width_pc=100, 
                 height_pc=100, width_mm=0, height_mm=0, ratio_lock='width', 
                 ratio=0.0, x_pc=0, y_pc=0, x_mm=0, y_mm=0):
      
        """
        text (str) -- text to be displayed in the button
        animate (bool) -- wether widget should be animated
        
        ... (*) -- refer to glitter.Container for documentation
        """
        
        Container.__init__(self, 
                           width_pc=width_pc,
                           height_pc=height_pc,
                           width_mm=width_mm,
                           height_mm=height_mm,
                           ratio_lock=ratio_lock,
                           ratio=ratio,
                           x_pc=x_pc,
                           y_pc=y_pc,
                           x_mm=x_mm,
                           y_mm=y_mm)
        
        self.text = text
        self.animate = animate   
             
        self.background = None
        self.icon = None
        
        # Setting reactive registers clutter events related to the Actor
        self.set_reactive(True)
        
        self.connect('motion-event', self.do_motion_event)
        self.connect('leave-event', self.do_leave_event)
        self.connect('button-press-event', self.do_press_event)
        self.connect('button-release-event', self.do_release_event)

    # Property getters
    def get_text(self):
        """ Retrieve button text
        
        return (str)
        """
        
        return self.text
        
    def get_animate(self):
        """ Retrieve wether widget is animated
        
        return (bool)
        """
        
        return self.animate
        
    # Property setters
    def set_text(self, value):
        """ Set button text
        
        value (str)
        """
        
        self.text = value
             
    def set_animate(self, value):
        """ Set wether widget is animated
        
        value (bool)
        """
        
        self.animate = value
    
    # Handle events
    def do_motion_event(self, stage, event):
        """ Pointer located in the widget area """
        
        self.background.hide()
        self.hover.show()
        
    def do_leave_event(self, stage, event):
        """ Pointer left widget area """
        
        self.press.hide()
        self.hover.hide()
        self.background.show()
        
    def do_press_event(self, stage, event):
        """ Pointer pressed inside the widget area """
        
        self.background.hide()
        self.hover.hide()
        self.press.show()
        self.emit('pressed', 'pressed')
        
    def do_release_event(self, stage, event):
        """ Pointer released """
        
        self.press.hide()
        self.hover.show()
        self.emit('clicked', 'clicked')
        self.emit('released', 'released')

    def update(self):
        """ Update widget """   
             
        super(Button, self).update()

        # if background is not set we need to set visual elements
        if not self.background:
            self._set_elements()
        else:
            self.background.update()
            self.hover.update()
            self.press.update()
            self.icon.update()
            self.label.update()

    def _set_elements(self):
        """ Set visual elements """

        # Button background texture
        self.background = Image(source='button_background_texture.py')
        self.add(self.background)
        self.background.show()
        
        # Button texture when hovered
        self.hover = Image(source='button_hover_texture.py')
        self.add(self.hover)
        self.hover.show()
        self.hover.hide()
        
        # Button texture when pressed
        self.press = Image(source='button_press_texture.py')
        self.add(self.press)
        self.press.show()
        self.press.hide()
        
        # Button icon
        self.icon = Image(source="test.svg", 
                          height_pc=80, 
                          ratio_lock='height', 
                          ratio=1.0)
        self.add(self.icon)
        self.icon.set_x_pc(2)
        self.icon.set_y_pc(10)
        self.icon.show()
        
        # Button label
        self.label = Label(text=self.text, width_pc=80)
        self.add(self.label)
        self.label.set_x_pc(25)
        self.label.show()
        
        if self.animate:
            # Fade in all elements (time: 500ms)
            self.appear_timeline = clutter.Timeline(duration=500)
            self.appear_alpha = clutter.Alpha(self.appear_timeline,
                                              clutter.smoothstep_inc_func)
            self.appear_behaviour = clutter.BehaviourOpacity(
                                        alpha=self.appear_alpha,
                                        opacity_start=0,
                                        opacity_end=255)
            self.appear_behaviour.apply(self.background)
            self.appear_behaviour.apply(self.label)
            self.appear_behaviour.apply(self.icon)
            self.appear_timeline.start()

    __gsignals__ = {
        'clicked': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
                   (gobject.TYPE_STRING,)),
        'pressed': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
                   (gobject.TYPE_STRING,)),
        'released': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
                   (gobject.TYPE_STRING,))
        }
        
        
gobject.type_register(Button)
