# !/usr/bin/python
# -*- coding: utf-8 -*-

# Glitter Toolkit

__authors__ = ["Jan Jokela <janjokela@gmail.com>"]
__licenses__ = ["LICENSE.LGPL"]
__description__ = "Container widget" 

import gobject
import clutter

class Container(clutter.Group):

    """ Glitter container widget """

    def __init__(self, width_pc=100, height_pc=100, width_mm=0, height_mm=0, 
                 ratio_lock='height', ratio=0.0, x_pc=0, y_pc=0, x_mm=0, 
                 y_mm = 0):

        """
        width_pc (int) -- width as a percentage of parent actor.
        height_pc (int) -- height as a percentage of parent actor.
        width_mm (int) -- width in millimeters. if width as a percentage is 
                          set, this will act as a minimum boundary.
        height_mm (int) -- height in millimeters. if height as a percentage is
                           set, this will act as a minimum boundary.
        ratio_lock (str) -- locks ratio to "width" or "height".
        ratio (float) -- if ratio > 0.0, a ratio will be applied depending on
                         ratio_lock.
        x_pc (int) -- x position related to parent actor as a percentage of the
                      former.
        y_pc (int) -- y position relates to parent actor as a percentage of the
                      former.
        x_mm (int) -- x position related to parent actor in millimeters, if
                      x_pc is set, this will act as a minimum boundary.
        y_mm (int) -- y position related to parent actor in millimeters, if
                      y_pc is set, this will act as a minimum boundary.
        """
        
        clutter.Group.__init__(self)
                         
        self.width_pc = width_pc
        self.height_pc = height_pc
        self.width_mm = width_mm
        self.height_mm = height_mm
        self.ratio_lock = ratio_lock
        self.ratio = ratio
        self.x_pc = x_pc
        self.y_pc = y_pc
        self.x_mm = x_mm
        self.y_mm = y_mm
        
        self.stage = clutter.Stage()
        
        self.connect('parent-set', self.do_parent_set)
        self.stage.connect('notify::width', self.do_resize)
        self.stage.connect('notify::height', self.do_resize)
    
    # Property getters    
    def get_width_pc(self):
        """ Retrieve width as a percentage of parent actor 
        
        return (int)
        """
        
        return self.width_pc
        
    def get_height_pc(self):
        """ Retrieve height as a percentage of parent actor 
        
        return (int)
        """
        
        return self.height_pc
        
    def get_width_mm(self):
        """ Retrieve width in millimeters 
        
        return (int)
        """
        
        return self.width_mm
        
    def get_height_mm(self):
        """ Retrieve height in millimeters 
        
        return (int)
        """
        
        return self.height_mm
        
    def get_width_px(self):
        """ Retrieve width as pixels 
        
        return (int)
        """
        
        return self.width_px
        
    def get_height_px(self):
        """ Retrieve height as pixels 
        
        return (int)
        """
        
        return self.height_px
        
    def get_ratio_lock(self):
        """ Retrieve ratio lock
        
        return (str) -- width or height
        """
    
        return self.ratio_lock
        
    def get_ratio(self):
        """ Retrieve ratio (depends on ratio_lock)
        
        return (float)
        """
        
        return self.ratio
        
    def get_x_pc(self):
        """ Retrieve x position as a percentage of parent actor 
        
        return (int)
        """
        
        return self.x_pc
        
    def get_y_pc(self):
        """ Retrieve y position as a percentage of parent actor
        
        return (int)
        """
    
        return self.y_pc
        
    def get_x_mm(self):
        """ Retrieve x position in millimeters from parent actor anchor
        
        return (int)
        """
    
        return self.x_mm
        
    def get_y_mm(self):
        """ Retrieve y position in millimeters from parent actor anchor
        
        return (int)
        """
    
        return self.y_mm
        
    # Property setters
    def set_width_pc(self, value):
        """ Set width as a percentage of parent actor
        
        value (int)
        """
        
        self.width_pc = value
        self.update()
    
    def set_height_pc(self, value):
        """ Set height as a percentage of parent actor
        
        value (int)
        """
        
        self.height_pc = value
        self.update()

    def set_width_mm(self, value):
        """ Set width in millimeters 
        
        value (int)
        """
        
        self.width_mm = value
        self.update()
        
    def set_height_mm(self, value):
        """ Set height in millimeters
        
        value (int)
        """
        
        self.height_mm = value
        self.update()
        
    def set_width_px(self, value):
        """ Set width in pixels 
        
        value (int)
        """
        
        self.width_px = value
        
    def set_height_px(self, value):
        """ Set height in pixels 
        
        value (int)
        """
        
        self.height_px = value
        
    def set_ratio_lock(self, value):
        """ Set ratio lock to width or height
        
        value (str) -- width or height 
        """
        
        self.ratio_lock = value
        
    def set_ratio(self, value):
        """ Set ratio, depends on ratio_lock
        
        value (float)
        """
        
        self.ratio = value
        
    def set_x_pc(self, value):
        """ Set x position as a percentage of parent actor
        
        value (int)
        """
        
        self.x_pc = value
        self._set_x()
        
    def set_y_pc(self, value):
        """ Set y position as a percentage of parent actor
        
        value (int)
        """      
     
        self.y_pc = value
        self._set_y()
        
    def set_x_mm(self, value):
        """ Set x in millimeters from parent actor anchor
        
        value (int)
        """
        
        self.x_mm = value
        self._set_x()
        
    def set_y_mm(self, value):
        """ Set y in millimeters from parent actor anchor
        
        value (int)
        """
        
        self.y_mm = value
        self._set_y()
    
    # Property definitions
    # FIXME: Properties with custom getter/setter don't support inheritance due
    # to a bug in pygobject. (fixed, but fix not available in most deployments)
    # If gobject properties are to be used, getters and setters must refer
    # to self._property_name (we can't use the same variable name)
    """
    width_pc = gobject.property(
                 type=int, getter=get_width_pc, setter=set_width_pc
               )
    height_pc = gobject.property(
                  type=int, getter=get_height_pc, setter=set_height_pc
                )
    width_mm = gobject.property(
                 type=int, getter=get_width_mm, setter=set_width_mm
               )
    height_mm = gobject.property(
                  type=int, getter=get_height_mm, setter=set_height_mm
                )
    ratio_lock = gobject.property(
                   type=str, getter=get_ratio_lock, setter=set_ratio_lock
                 )
    ratio = gobject.property(
              type=float, getter=get_ratio, setter=set_ratio
            )
    x_pc = gobject.property(
             type=int, getter=get_x_pc, setter=set_x_pc
           )
    y_pc = gobject.property(
             type=int, getter=get_y_pc, setter=set_y_pc
           )
    x_mm = gobject.property(
             type=int, getter=get_x_mm, setter=set_x_mm
           )
    y_mm = gobject.property(
             type=int, getter=get_y_mm, setter=set_y_mm
           )
    """

    def px_to_width_pc(self, px):
        """ Retrieve the width percentage correspondent to given pixels
        
        px (int) -- pixels, pixel count starts at widget anchor
        return (int) -- the correspondent percentage
        """
        
        return int(px * 100 / self.get_width_px())

    def px_to_height_pc(self, px):
        """ Retrieve the height percentage correspondent to given pixels
        
        px (int) -- pixels, pixel count starts at widget anchor
        return (int) -- the correspondent percentage
        """
        
        return int(px * 100 / self.get_height_px())

    # Event handlers
    def do_resize(self, stage, event):
        """ Handles stage resize events """
        
        self.update()
       
    def do_parent_set(self, stage, event):
        """ When we change parent, we need to update the widget """

        self.update()
        
    def update(self):
        """ Updates the container 
        
        only position will be set here. changes in width and height are handled
        by inheriting widgets.
        """        

        #print "Container.update " + str(self) 
        self._set_size()
        self._set_x()
        self._set_y()

    # Misc
    def _set_size(self):
        """ Calculates Container size (width and height) in pixels based on
        those values as percentages and millimeters and ratios
        """
        
        width_as_mm = clutter.units_to_device(
                        clutter.units_from_mm(self.get_width_mm()) 
                      )
        height_as_mm = clutter.units_to_device(
                         clutter.units_from_mm(self.get_height_mm())
                       )
        width_as_pc = self._units_from_parent_width_percentage(self.get_width_pc())
        height_as_pc = self._units_from_parent_height_percentage(self.get_height_pc())
        
        if self.width_pc == 0:
            width = width_as_mm
        else:
            if width_as_pc < width_as_mm:
                width = width_as_mm
            else: 
                width = width_as_pc
        
        if self.height_pc == 0:
            height = height_as_mm
        else: 
            if height_as_pc < height_as_mm:
                height = height_as_mm
            else:
                height = height_as_pc
                
        if self.ratio != 0.0:
            if self.ratio_lock == 'width':
                height = width * self.ratio
            elif self.ratio_lock == 'height':
                width = height * self.ratio
        
        self.set_width_px(width)
        self.set_height_px(height)

    def _set_x(self):
        """ Sets x position of Container based on x position in percentages and
        millimeters """
        
        x_pc = self._units_from_parent_width_percentage(self.get_x_pc())   
        x_mm = clutter.units_to_device(
                 clutter.units_from_mm(self.get_x_mm())
               )
        if self.x_pc == 0:
            x = x_mm
        else:
            if x_pc < x_mm:
                x = x_mm
            else: 
                x = x_pc
        self.set_x(x)

    def _set_y(self):
        """ Sets y position of Container based on y position in percentages and
        millimeters """
        
        y_pc = self._units_from_parent_height_percentage(self.get_y_pc())   
        y_mm = clutter.units_to_device(
                 clutter.units_from_mm(self.get_y_mm())
               )
        if self.y_pc == 0:
            y = y_mm
        else:
            if y_pc < y_mm:
                y = y_mm
            else: 
                y = y_pc
        self.set_y(y)

    def _units_from_parent_width_percentage(self, percent):
        """ Returns the number of pixels correspondant to % of screen width
        
        return (integer) -- number of correspondant pixels in x axis
        """

        if self.get_parent() and self.get_parent() != self.stage:
            return int(float(self.get_parent().get_width_px()) * percent / 100)
        else:
            return int(float(self.stage.get_width()) * percent / 100)
 
    def _units_from_parent_height_percentage(self, percent):
        """ Returns the number of pixels correspondant to % of screen height
        
        return (integer) -- number of correspondant pixels in y axis
        """
        
        if self.get_parent() and self.get_parent() != self.stage:
            return int(float(self.get_parent().get_height_px()) * percent / 100)
        else:
            return int(float(self.stage.get_height()) * percent / 100)

   
gobject.type_register(Container)

