# !/usr/bin/python
# -*- coding: utf-8 -*-

# Glitter Toolkit

__authors__ = ["Jan Jokela <janjokela@gmail.com>"]
__licenses__ = ["LICENSE.LGPL"]
__description__ = "Image widget" 

import os
import sys

import gobject
import clutter

from container import Container
from pixbuf_image import PixbufImage
from svg_image import SVGImage

class Image(Container):
    """ Glitter image widget """

    def __init__(self, source='default_cairo_texture.py', 
                 source_scalable=True, width_pc=100, height_pc=100, 
                 width_mm=0, height_mm=0, ratio_lock='width', ratio=0.0,
                 x_pc=0, y_pc=0, x_mm=0, y_mm=0):
        """
        source (str) -- image source:
                        - *.png (a bitmap)
                        - *.svg (a scalable vector graphic)
                        - *.py (a python file containing a "Texture" class 
                          inheriting from clutter.cluttercairo.CairoTexture)
        source_scalable (bool) -- if source is scalable (ex: svg or 
                                  CairoTexture), source will be recomputed 
                                  instead of scaled
        
        .. (*) -- refer to glitter.Container for documentation
        """
                
        Container.__init__(self, 
                           width_pc=width_pc, 
                           height_pc=height_pc, 
                           width_mm=width_mm, 
                           height_mm=height_mm, 
                           ratio_lock=ratio_lock, 
                           ratio=ratio,
                           x_pc=x_pc,
                           y_pc=y_pc,
                           x_mm=x_mm,
                           y_mm=y_mm)
    
        self.source = source
        self.source_scalable = source_scalable
        
        self.texture = None
    
    # Property getters    
    def get_source(self):
        """ Retrieves image source file uri 
        
        return (str) -- image source uri in standard uri format
        """
        
        return self.source
        
    def get_source_scalable(self):
        """ Retrieves wether image source file should be handled as a scalable 
        graphic (only applicable to SVGs and Cairo textures) 
        
        return (bool)
        """
        
        return self.source_scalable
        
    # Property setters
    def set_source(self, value):
        """ Sets image source uri 
        
        value (str) -- image source uri in standard uri format
        """
        
        self.source = value
        
    def set_source_scalable(self, value):
        """ Sets wether image source file should be handled as a scalable 
        graphic (only applicable to SVGs and Cairo textures) 
        
        value (bool)
        """
    
        self.source_scalable = value
           
    # Property definitions
    # FIXME: Properties with custom getter/setter don't support inheritance due
    # to a bug in pygobject. (fixed, but fix not available in most deployments)
    """source = gobject.property(
               type=str, getter=get_source, setter=set_source
             )
    source_scalable = gobject.property(
                        type=bool, 
                        default=True,
                        getter=get_source_scalable, 
                        setter=set_source_scalable
                      )
    """

    def update(self):
        """ Updates actor """
        
        super(Image, self).update()
        
        #print "image y is: " + str(self.get_y())

        if not self.texture or self.source_scalable:
            self._set_source()
        else:
            width = self.get_width_px()
            height = self.get_height_px()
            self.texture.set_width(width)
            self.texture.set_height(height)
     
    def _set_source(self):
        """ Set texture based on given source
        
        - current support for PNG, SVG and Cairo
        """
        
        width = self.get_width_px() 
        height = self.get_height_px()
        
        source = os.path.splitext(self.source)[0]
        source_ext = os.path.splitext(self.source)[1]
        
        if source_ext == '.py':
            source_path = os.path.join(sys.path[0], "data", "themes", "nublo")
        else:
            source_path = os.path.join(sys.path[0], "data", "themes", "nublo")
        
        if os.path.exists(source_path):
            if source_ext == '.png':
                self.texture = PixbufImage(
                                 os.path.join(source_path, self.source)
                               )
                self.texture.set_width(width)
                self.texture.set_height(height)
            elif source_ext == '.svg':
                if self.texture:
                    self.remove(self.texture)
                    del(self.texture)
                self.texture = SVGImage(os.path.join(source_path, self.source), width, height)
            elif source_ext == '.py':
                if self.texture:
                    self.remove(self.texture)
                    del(self.texture)
                source_import = os.path.basename(source)
                sys.path[1] = os.path.join(sys.path[0], "data", "themes", "nublo")
                cairotexture = __import__(source_import)
                self.texture = cairotexture.Texture(width, height)
            self.add(self.texture)
            self.texture.show()
        else: 
            raise AttributeError, "source doesn't exist: %s" % source_path
            
               
gobject.type_register(Image)

