# !/usr/bin/python
# -*- coding: utf-8 -*-

# Glitter Toolkit

__authors__ = ["Jan Jokela <janjokela@gmail.com>"]
__licenses__ = ["LICENSE.LGPL"]
__description__ = "Slider widget" 

import gobject
import clutter

from container import Container
from image import Image

class Slider(Container):
    """ Glitter slider widget """

    def __init__(self, percentage=0, width_pc=100, height_pc=100, width_mm=0,
                 height_mm=0, ratio_lock='width', ratio=0.0, x_pc=0, y_pc=0, 
                 x_mm=0, y_mm=0):
        
        """ 
        percentage (int) -- slider percentage [0 .. 100]
        
        * -- refer to Container for inherited arguments 
        """
        
        Container.__init__(self, 
                           width_pc=width_pc,
                           height_pc=height_pc,
                           width_mm=width_mm,
                           height_mm=height_mm,
                           ratio_lock=ratio_lock,
                           ratio=ratio,
                           x_pc=x_pc,
                           y_pc=y_pc,
                           x_mm=x_mm,
                           y_mm=y_mm)
                           
        self.percentage = percentage
        
        self.unfilled_texture = None
        self.filled_texture = None
        self.handle_texture = None
        
        self.set_reactive(True)
        
        self.stage = clutter.Stage()
        
        self.stage.connect('motion-event', self.do_motion_event)
        self.connect('leave-event', self.do_leave_event)
        #self.connect('button-press-event', self.do_press_event)
        self.stage.connect('button-release-event', self.do_release_event)
        
        self.handle_locked = False

    # Property getters
    def get_percentage(self):
        """ Retrieve slider percentage
        
        return (int) -- [0 .. 100]
        """
        
        return self.percentage
        
    # Property setters
    def set_percentage(self, value):
        """ Set slider percentage
        
        value (int) -- slider percentage [0 .. 100]
        """
        
        self.percentage = value
        self.update()
        self.emit('value-changed', self.percentage)
        
    # Event handlers
    def do_motion_event(self, stage, event):
        """ Cursor is in the widget area """
        
        if self.handle_locked:
            # as event.x returns the cursor in pixels, we need to discover to
            # what percentage of our slider does it correspond
            x_px_from_anchor = event.x - self.get_x()
            pos_as_width_pc = self.px_to_width_pc(x_px_from_anchor)
            # We don't want drag the handle out of the widget
            if pos_as_width_pc > 0 and pos_as_width_pc < 100:
                #self.handle_texture.set_x_pc(pos_as_width_pc)
                self.set_percentage(pos_as_width_pc)
            elif pos_as_width_pc < 0:
                #self.handle_texture.set_x_pc(0)
                self.set_percentage(0)
            elif pos_as_width_pc > 100:
                self.set_percentage(100)
                #self.handle_texture.set_x_pc(100)
               
        
    def do_leave_event(self, stage, event):
        """ Cursor left the widget area """
        
        pass
        
    def do_press_event(self, stage, event):
        """ Pointer pressed inside the widget area """
        
        self.handle_locked = True
        
    def do_release_event(self, stage, event):
        """ Pointer released anywhere on the stage """
        
        if self.handle_locked:
            self.handle_locked = False

    # Misc
    def update(self):
        """ Update widget """
        
        super(Slider, self).update()
        
        self._set_elements()

    def _set_elements(self):
        
        if not self.unfilled_texture:
            self.unfilled_texture = Image(source="slider_unfilled_texture.py", 
                                          height_pc=60,
                                          y_pc=20)
            self.add(self.unfilled_texture)
            self.unfilled_texture.show()
            
        if not self.filled_texture:
            self.filled_texture = Image(source="slider_filled_texture.py",
                                        width_pc=self.percentage,
                                        height_pc=60,
                                        y_pc=20)
            self.add(self.filled_texture)
            self.filled_texture.show()

        else:
            
            self.filled_texture.set_width_pc(self.get_percentage())
            
        if not self.handle_texture:
            
            self.handle_texture = Image(source="slider_handle_texture.py",
                                        ratio_lock="height", 
                                        ratio=1.0)
            self.add(self.handle_texture)
            x_pc = self.px_to_width_pc(
                     int(self.handle_texture.get_height_px() / 2.0)
                   )
            x_pc = self.percentage - x_pc
            self.handle_texture.set_x_pc(x_pc)
            self.handle_texture.show()
            self.handle_texture.set_reactive(True)
            self.handle_texture.connect('button-press-event', self.do_press_event)
            
        else:
            x_pc = self.px_to_width_pc(
                     int(self.handle_texture.get_height_px() / 2.0)
                   )
            x_pc = self.percentage - x_pc
            self.handle_texture.set_x_pc(x_pc) 
            
    __gsignals__ = {
        'value-changed': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
                             (gobject.TYPE_INT,))   
    }
   
gobject.type_register(Slider)

