# !/usr/bin/python
# -*- coding: utf-8 -*-

# Glitter Toolkit

__authors__ = ["Jan Jokela <janjokela@gmail.com>"]
__licenses__ = ["LICENSE.LGPL"]
__description__ = "Toggle switch widget"

import gobject

from container import Container
from image import Image
from label import Label

class ToggleSwitch(Container):

    """ Glitter toggle switch """
    
    def __init__(self, state=True, animate=True, width_pc=100, height_pc=100, 
                 width_mm=0, height_mm=0, ratio_lock='width', x_pc=0, y_pc=0, 
                 x_mm=0, y_mm=0,):
      
        """
        ... (*) -- refer to Container for documentation
        """
        
        if ratio_lock == 'width': ratio = 0.5
        else: ratio = 3.0
      
        Container.__init__(self, 
                           width_pc=width_pc,
                           height_pc=height_pc,
                           width_mm=width_mm,
                           height_mm=height_mm,
                           ratio_lock=ratio_lock,
                           ratio=ratio,
                           x_pc=x_pc,
                           y_pc=y_pc,
                           x_mm=x_mm,
                           y_mm=y_mm)
                           
        self.state = state
        self.animate = animate
                           
        self.background = None
        
        self.set_reactive(True)
        self.motion_locked = False
        
        self.connect('motion-event', self.do_motion_event)
        self.connect('leave-event', self.do_leave_event)
        self.connect('button-press-event', self.do_press_event)
        self.connect('button-release-event', self.do_release_event)
             
    # Property getters
    def get_state(self):
        """ Retrieve switch state (on/off)
        
        return (bool)
        """
        
        return self.state
        
    def get_animate(self):
        """ Retrieve wether animations are enabled 
        
        return (bool)
        """
        
        return self.animate
        
    # Property setters
    def set_state(self, value):
        """ Set switch state (on/off)
        
        value (bool)
        """
        
        self.state = value
        
    def set_animate(self, value):
        """ Enable or disable animations

        value (bool)
        """
        
        self.animate = value

    # Event handling
    def do_press_event(self, stage, event):
        self.motion_locked = True
        self.motion_xpos = event.x - self.get_x() - self.selection.get_x()
        self.selection.hide()
        self.selection_press.show()
    
    def do_release_event(self, stage, event):
        self.motion_locked = False
        self.selection.show()
        self.selection_press.hide()
        if float(event.x - self.get_x()) * 100 / float(self.get_width_px()) < 50:
            self.state = False
            self.enabled_label.hide()
            self.disabled_label.show()
            self.selection.set_x_pc(0)
            self.selection_press.set_x_pc(0)
        else:
            self.disabled_label.hide()
            self.enabled_label.show()
            self.state = True
            self.selection.set_x_pc(60)
            self.selection_press.set_x_pc(60)
            
            
    def do_motion_event(self, stage, event):
        if self.motion_locked:
            position = event.x - self.get_x() - self.motion_xpos
            if position >= 0 and position <= self.get_width_px():
                self.selection_press.set_x(event.x - self.get_x() - self.motion_xpos)
    
    def do_leave_event(self, stage, event):
        if self.motion_locked:
            self.motion_locked = False
            if float(event.x - self.get_x()) * 100 / float(self.get_width_px()) < 50:
                self.state = False
                self.enabled_label.hide()
                self.disabled_label.show()
                self.selection.set_x_pc(0)
                self.selection_press.set_x_pc(0)
            else:
                self.disabled_label.hide()
                self.enabled_label.show()
                self.state = True
                self.selection.set_x_pc(60)
                self.selection_press.set_x_pc(60)    

    def update(self):
        """ """
        
        super(ToggleSwitch, self).update()

        if not self.background:
            self._set_elements()
                           
    def _set_elements(self):
        """ Set visual elements """
        
        self.background = Image(source='toggle_background_texture.py')
        self.add(self.background)
        self.background.show()    
        
        self.enabled_label = Label(text="On",
                                   color=(200, 200, 200, 255),
                                   height_pc=80)
        self.add(self.enabled_label)
        self.enabled_label.set_x_pc(10)
        self.enabled_label.set_y_pc(10)
        self.enabled_label.show()
        
        self.disabled_label = Label(text="Off",
                                    color=(200, 200, 200, 255),
                                    height_pc=80)
        self.add(self.disabled_label)
        self.disabled_label.set_x_pc(50)
        self.disabled_label.set_y_pc(10)
        self.disabled_label.show()
        
        self.selection = Image(source='toggle_selection_texture.py',
                               width_pc=40)
        self.add(self.selection)
        self.selection.show()         
              
        self.selection_press = Image(source='toggle_selection_press_texture.py',
                           width_pc=40)
        self.add(self.selection_press)
        self.selection_press.hide()
        self.selection_press.show()
                                       
    
gobject.type_register(ToggleSwitch)
