# !/usr/bin/python
# -*- coding: utf-8 -*-

# Glitter Toolkit

__authors__ = ["Jan Jokela <janjokela@gmail.com>", "Lauri Taimila <lauri@taimila.com>"]
__licenses__ = ["LICENSE.LGPL"]
__description__ = "Video widget" 

import os
import sys
import math

import gst
import gobject
import clutter
from clutter import cluttergst

from container import Container

class Video(Container):
    """ Glitter video widget """

    def __init__(self, source, aspect_ratio="native", width_pc=100, height_pc=100, 
                 width_mm=0, height_mm=0, ratio_lock='width', ratio=0.0,
                 x_pc=0, y_pc=0, x_mm=0, y_mm=0):
                    
        """
        source (str) -- video source:
                        - *.avi/mpeg/mpg/mov
        aspect_ratio (str) -- video ratio:
                              - native, widescreen, tv, smart
        
        .. (*) -- refer to glitter.Container for documentation
        """
                
        Container.__init__(self, 
                           width_pc=width_pc, 
                           height_pc=height_pc, 
                           width_mm=width_mm, 
                           height_mm=height_mm, 
                           ratio_lock=ratio_lock, 
                           ratio=ratio,
                           x_pc=x_pc,
                           y_pc=y_pc,
                           x_mm=x_mm,
                           y_mm=y_mm)
    
        self.source = source
        self.aspect_ratio = aspect_ratio
        
        self.texture = None
    
    # Property getters    
    def get_source(self):
        """ Retrieve video source file uri 
        
        return (str) -- video source uri in standard uri format
        """
        
        return self.source
        
    def get_aspect_ratio(self):
        """ Retrieve video aspect ratio
        
        return (str) -- "native", "widescreen", "tv", "smart"
        """
        
        return self.aspect_ratio
        
    def get_volume(self):
        """ Retrieve volume
        
        return (float) -- volume [0.0 .. 1.0]
        """
        
        return self.texture.get_volume()
        
    def get_position(self):
        """ Retrieve stream position
        
        return (float) -- position [0.0 .. 1.0]
        """
        
        try:
            pos = self.playbin.query_position(gst.FORMAT_TIME, None)[0]
            dur = self.playbin.query_duration(gst.FORMAT_TIME, None)[0]
        except gst.QueryError, ex:
            #this normally means that the MediaPlayer object is querying
            #before the media is playing
            return 0.0
        except Exception, ex:
            print ex.message
            return 0.0
        dur_sec = dur / 1000000000.0
        pos_sec = pos / 1000000000.0
        return pos_sec / dur_sec
        
    def get_duration(self):
        """ Retreive video duration
        
        return (int)
        """
        
        return self.texture.get_duration()
        
    # Property setters
    def set_source(self, value):
        """ Sets video source uri 
        
        value (str) -- video source uri in standard uri format
        """
        
        self.source = value
        self.texture = None
        self.update()

    def set_aspect_ratio(self, value):
        """ Sets video aspect ratio
        
        value (str) -- video aspect ratio "native", "widescreen", "tv" or 
                       "smart"
        """
        
        self.aspect_ratio = value
        self.update()
        
    def set_volume(self, value):
        """ Sets video volume
        
        value (float) -- volume [0.0 .. 1.0]
        """
        
        self.texture.set_volume(value)
        
    def set_position(self, value):
        """ Sets position
        
        value (int) -- [>= 0]
        """
        
        self.texture.set_position(value)
        
    def play(self):
        """ Play video. Resumes or starts video playback """
        
        if not self.is_playing():
            self.texture.set_playing(True)
    
    def pause(self):
        """ Pause video playback """
        
        if self.is_playing():
            self.texture.set_playing(False)
    
    def stop(self):
        """ Stop video playback """
        
        if self.texture:
            self.remove(self.texture)
            self.texture.set_playing(False)
            self.texture.set_property("position", 0)
            
    def is_playing(self):
        """ Retrieve wether video is playing """
    
        if self.texture:
            if self.texture.get_playing():
                return True
        return False

    def update(self):
        """ Updates actor """
        
        super(Video, self).update()

        if not self.texture:
            self._set_source()
            
        width = self.get_width_px()
        height = self.get_height_px()

        width_delta = math.fabs(width - width * 0.5625)
        height_delta = math.fabs(height - height * 1.7777)        
        
        # native
        if self.aspect_ratio == "native":
            self.texture.set_x(0)
            self.texture.set_y(0)
            self.texture.set_width(width)
            self.texture.set_height(height)
        
        # 16:9 widescreen (720p & 1080p)
        elif self.aspect_ratio == "widescreen":
            if width_delta < height_delta:
                video_height = int(width * 0.5625)
                self.texture.set_width(width)
                self.texture.set_height(video_height)
                self.texture.set_x(0)
                self.texture.set_y(int((height - video_height) / 2.0))
            else:
                video_width = int(height * 1.7777)
                self.texture.set_width(video_width)
                self.texture.set_height(height)
                self.texture.set_x(int((width - video_width) / 2.0))
                self.texture.set_y(0)
        
        # 4:3 tv-like        
        elif self.aspect_ratio == "tv":
            if width_delta < height_delta:
                video_height = int(width * 0.75)
                self.texture.set_width(width)
                self.texture.set_height(video_height)
                self.texture.set_x(0)
                self.texture.set_y(int((height - video_height) / 2.0))
            else:
                video_width = int(height * 1.3333)
                self.texture.set_width(video_width)
                self.texture.set_height(height)
                self.texture.set_x(int((width - video_width) / 2.0))
                self.texture.set_y(0)            
            
        # smart: maximum viewing area without cutting too much        
        elif self.aspect_ratio == "smart":
            if width_delta < height_delta:
                video_height = int(width * 0.5625)
                self.texture.set_width(width)
                self.texture.set_height(video_height)
                self.texture.set_x(0)
                self.texture.set_y(int((height - video_height) / 2.0))
            else:
                video_width = int(height * 1.7777)
                self.texture.set_width(video_width)
                self.texture.set_height(height)
                self.texture.set_x(int((width - video_width) / 2.0))
                self.texture.set_y(0)            
    
    def _on_gst_message(self, bus, message):
        """ Messages from our gstreamer bus """
        
        if message.type == gst.MESSAGE_EOS:
            self.stop()
        elif message.type == gst.MESSAGE_ERROR:
            print "Gstreamer exception -- bus: " + str(bus) + ", message: " + \
                  str(message)
     
    def _set_source(self):
        """ Set texture based on given source """
        
        width = self.get_width_px() 
        height = self.get_height_px()
        
        if os.path.exists(self.source):
            self.texture = cluttergst.VideoTexture()
            self.playbin = self.texture.get_playbin()
            self.bus = self.playbin.get_bus()
            self.bus.add_signal_watch()
            self.bus.connect('message', self._on_gst_message)
            self.texture.show()
            self.texture.set_filename(self.source)
            self.add(self.texture)
            self.play()
        else: 
            raise AttributeError, "source doesn't exist: %s" % self.source
            
               
gobject.type_register(Video)

